import os
import django


#  you have to set the correct path to you settings module
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "email_proj.settings")
django.setup()

from .models import Subscriber, EmailTemplate

class EmailController():
    def __init__(self):
        pass

    def get_subscribers(self):
        subscribers = Subscriber.objects.all()
        for sub in subscribers:
            print(sub)

    def get_latest_email_template(self):
        pass


email_ctl = EmailController()
email_ctl.get_subscribers()


from django.urls import path
from .views import SubscribeView
from . import views
app_name = "email_app"
urlpatterns = [
    path("send_mail/", views.send_email, name="send_mail"),
    path("subscribe/", SubscribeView.as_view(), name="subscribe"),
]



from django.test import TestCase
from .models import Subscriber
from django.contrib.auth import get_user_model, authenticate, login
from django.utils import timezone
from django.urls import reverse

User = get_user_model()

def create_user_and_login():
    new_user = User.objects.create(username="test_username", password="test_password", email="test_email")
    new_user.save()
    user = authenticate(username="test_username", password="test_password")
    if user is not None:
        login(user=user)


class SubscriberModelTests(TestCase):

    def test_creates_subscriber(self):
        """  
        create subscriber return true if subscriber is created successfully
        """
        user = User.objects.create(username="test_username", password="test_password", email="test_email")

        subscribed_user = Subscriber(user=user, is_subscribed=True, subscribed_date=timezone.now())
        subscribed_user.save()
        subscribed_user.refresh_from_db()

        subscriber_data = Subscriber.objects.get(pk=1)
        
        self.assertEqual(subscriber_data.is_subscribed, True)



    def test_subscribe_view_get_template(self):
        """
        get subcribe view response, return true if repsonse code is 200
        """
        response = self.client.get(reverse("email_app:subscribe"))
        return self.assertEqual(response.status_code, 200)


import os
from dotenv import load_dotenv
from django.shortcuts import render,HttpResponse, HttpResponseRedirect
from django.contrib.auth import get_user_model
from django.views import View
from django.urls import reverse
from django.utils import timezone
from django.views.decorators.csrf import csrf_protect
from django.utils.decorators import method_decorator
from django.core.mail import send_mail
from .forms import SubscribeForm
from .models import Subscriber, EmailTemplate

load_dotenv()

class SubscribeView(View):
    template_name = "subscribe_form.html"
    form = SubscribeForm()
    context = {"form": form}

    def get(self, request):
        return render(request, self.template_name, self.context)

    @method_decorator(csrf_protect)
    def post(self, request):
        try:
            if request.POST["subscribe_check"] == "on":
                subscriber = Subscriber(user=request.user, is_subscribed=True, subscribed_date=timezone.now())
                subscriber.save()
                subscriber.refresh_from_db()
        except KeyError:
            pass
        finally:
            return HttpResponseRedirect(reverse("email_app:subscribe"))


def send_email(request):
    subscribers = Subscriber.objects.all()
    sub_name_list = []
    sub_list = []
    for sub in subscribers:
        if sub.is_subscribed:
            sub_list.append(str(sub.user.email))
            sub_name_list.append(str(sub.user.username))

    email_template = EmailTemplate.objects.order_by("email_template_added").last()
    email_subject = str(email_template.subject_text)
    email_body = str(email_template.body_text)
    
    for name in sub_name_list:
        new_email_body = email_body.replace("[name]", name)

    for sub in sub_list:
        
        send_mail(
            email_subject,
            new_email_body,
            "blogpad@zohomail.com",
            [sub],
            fail_silently=False,
            auth_user="blogpad@zohomail.com",
            auth_password=os.environ["EMAIL_PASSWORD"],
        )

    return HttpResponse(new_email_body)



from django.db import models
from django.contrib.auth import get_user_model
from django.utils import timezone

User = get_user_model()

class Subscriber(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    is_subscribed = models.BooleanField()
    subscribed_date = models.DateTimeField()
    

class EmailTemplate(models.Model):
    subject_text = models.CharField(max_length=50)
    body_text = models.TextField()
    email_template_added = models.DateTimeField("Date Added")
        